@@ -771,7 +771,7 @@ hildon_number_editor_focus                      (GtkWidget *widget,
   gboolean retval;
   GtkDirectionType effective_direction;
 
-  g_assert (HILDON_IS_EDITOR_EDITOR (widget));
+  g_assert (HILDON_IS_NUMBER_EDITOR (widget));
 
   retval = hildon_private_composite_focus (widget, direction, &effective_direction);
 
